// Copyright (c) 2002-2007 Telelogic AB
// All rights reserved.

/*
<trigger>
	<usage>bsf copy_attributes_to_associated_tasks.js attributes</usage>
	<version>5.0</version>
	<description>
		遷移の間に、属性値をある CR から関連タスクにコピーします。
	</description>
	<argument1>
		<name>attributes</name>
		<required>yes</required>
		<type>string</type>
		<description>空白で区切られた 1 つまたは複数の属性が、遷移の間に CR から関連タスクにコピーされます。
                             例えば、引数として "release priority" と指定すると、CR の release 属性と priority 属性が関連タスクにコピーされます。
					 このツールは、CR とタスクの間に同じ属性が定義されている場合にのみ有用です。
		</description>
	</argument1>
	<argument2>
		<name>-ifnotstate state_names</name>
		<required>no</required>
		<type>string</type>
		<description>1 つまたは空白で区切られた複数の状態名をチェックします。
					指定した状態の関連タスクには属性をコピーしません。
例えば、引数 "-ifnotstate completed excluded" は、完了タスクも除外されたタスクも修正しません。CR の他の完了していないタスクや除外されていないタスクは修正可能です。
		</description>
	</argument2>
</trigger>
*/
importPackage(Packages.com.telelogic.cs.api);

var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');

if (args.length < 1)
{
	log.error('Could not copy attributes to associated tasks: no attributes were given.');
	log.error('Usage: copy_attributes_to_associated_tasks.js attribute_name [attribute_name...] [-ifnotstate state1_to_skip [state2_to_skip...]]');
	log.error('Example: copy_attributes_to_associated_tasks.js resolver -ifnotstate completed');
}
else
{

	var allArgs = args;
	var attributesInArgs = allArgs;
	var skipStateSet;

	for (var i = 0; i < allArgs.length; i++)
	{
		if (allArgs[i].indexOf("-ifnotstate") == 0)
		{
			attributesInArgs = allArgs.slice(0, i);
			skipStateSet = allArgs.slice(i+1);
		}
	}
	
	// lookup the 'status' too if we want to skip tasks in a given state and need to compare it
	var attributeNames = skipStateSet ? ['status', 'from_db_path'] : ['from_db_path'];
	var tasks = change.getAssociatedTasks(event.triggerUser.token, event.objectId, attributeNames);

	if (tasks.length > 0)
	{
		var crAttributes = change.collectAttributes(event.triggerUser.token, event, attributesInArgs);

		var copiedAttributes = [];
		
		// gather only the attributes mentioned as args
		for (var i = 0; i < attributesInArgs.length; i++)
		{
			var name = attributesInArgs[i];
			var crAttribute = crAttributes.get(name);
			// gather that attribute if it was set on the CR. Otherwise, it's ignored
			if (crAttribute) copiedAttributes.push(crAttribute);
		}
		
		var doneTasks = {};
		var tasksToErrors = {};

		for (var i = 0; i < tasks.length; i++) 
		{
			var currentTask = tasks[i];
			
			var found = false;
			if (skipStateSet)
			{
				for (var j = 0; j < skipStateSet.length; j++)
				{
					if (currentTask.getAttribute('status').stringValue.equals(skipStateSet[j])) found = true;
				}
				if (found) 
				{
					continue;
				}
			}
			
			if (doneTasks[currentTask.taskId]) continue;

			// in central server mode, passing in a from_db_path will override the context database.
			// This lets up modify the task whereever it came from.
			var from = currentTask.getAttribute('from_db_path');

			var taskAttributes = from ?
				copiedAttributes.concat([from]) :
				copiedAttributes;

			try
			{
				change.setTaskAttributes(event.triggerUser.token, currentTask.taskId, taskAttributes);

				// the task transitioned in some database. Don't warn about it anymore. In central server mode
				// we may get DCM copies of tasks from multiple databases--we only need to transition the
				// task successfully in one of them (the modifiable_in DB).
				doneTasks[currentTask.taskId] = true;
				delete tasksToErrors[currentTask.taskId];
			}
			catch (error)
			{
				tasksToErrors[currentTask.taskId] =
					change.getMessage('couldNotCopyAttributesToTaskXErrorX', [currentTask.taskId, error.message || error.toString()]);
			}
		}
		
		for (var task in tasksToErrors)
		{
			event.fail(tasksToErrors[task]);
		}
	}
}