package ChangeSynergy::mrcActionsObjRc;

use strict;
use warnings;
use vars qw(); #this is the globals section
use ChangeSynergy::util; #The xmlDecode method exists in this class.
use XML::Lite; #We need this to actually do the XML parsing for us.

our $RC_ACTION_ADD		= "Add";
our $RC_ACTION_MOVE		= "Move";
our $RC_ACTION_UNDELETE	= "Undelete";
our $RC_ACTION_MODIFY		= "Modify";
our $RC_ACTION_DELETE		= "Delete";
our $RC_ACTION_PURGE		= "Purge";
our $RC_ACTION_REMOVE_OUTLINKS = "Remove outlinks";
our $RC_ACTION_CREATE_OUTLINKS = "Create outlinks";

sub new()
{
	shift;

	# Initialize data as an empty hash.
	my $self    = {};

	# Bless reference into class;
	bless $self;

	# Get the xml file and create the XML parsing object.
	my $xmlFile = shift;

	if(defined($xmlFile))
	{
		my $xml      = new XML::Lite($xmlFile);
		$self->{xml} = $xml;
	}
	else
	{
		$self->{xml} = undef;
	}

	$self->{rcID} = undef;
	$self->{parent_rcrID} = undef;
	$self->{doorsID} = undef;
	$self->{doors_object_path} = undef;
	$self->{doorsIsApplied} = undef;
	$self->{doorsIsRejected} = undef;
	$self->{modify_attrs} = undef;
	$self->{actions} = undef;
	$self->{below} = undef;
	$self->{after} = undef;
	$self->{siblings} = undef;
	$self->{linkdata} = undef;

	if(defined($self->{xml}))
	{
		&parseXml($self);
	}

	return $self;
}


sub get_rcID()
{
	my $self    = shift;
	return $self->{rcID};
}

sub get_parent_rcrID()
{
	my $self = shift;
	return $self->{parent_rcrID};
}

sub get_doorsID()
{
	my $self = shift;
	return $self->{doorsID};
}

sub get_doors_object_path()
{
	my $self = shift;
	return $self->{doors_object_path};
}

sub get_doorsIsApplied()
{
	my $self = shift;
	return $self->{doorsIsApplied}
}

sub get_doorsIsRejected()
{
	my $self = shift;
	return $self->{doorsIsRejected}
}

sub get_modify_attrs()
{
	my $self = shift;
	return $self->{modify_attrs};
}

sub has_modify_attr()
{
	my $self = shift;
	my $attrToFind = shift;
	my $modifiedAttrs = $self->{modify_attrs};
	foreach my $modifiedAttr (@$modifiedAttrs)
	{
		if ($modifiedAttr eq $attrToFind)
		{
			return 1 #TRUE
		}
	}
	return 0 #FALSE
	
}

sub get_actions()
{
	my $self = shift;
	return $self->{actions};
}

sub has_action()
{
	my $self = shift;
	my $actionToFind = shift;
	my $actions = $self->{actions};
	foreach my $action (@$actions)
	{
		if ($action eq $actionToFind)
		{
			return 1 #TRUE
		}
	}
	return 0 #FALSE
}


# Will only be defined if there is a move or add action
sub get_after()
{
	my $self = shift;
	return $self->{after};
}

# Will only be defined if there is a move or add action
sub get_below()
{
	my $self = shift;
	return $self->{below};
}

# Will only be defined if there is a move or add action
sub get_siblings()
{
	my $self = shift;
	return $self->{siblings};
}

# Will only be defined if there is a link action
sub get_linkdata()
{
	my $self = shift;
	return $self->{linkdata};
}

#
# Private Methods
#
sub parseXml
{
	my $self = shift;
	my @keys = $self->{xml}->elements_by_name("KEY");
	my @values = $self->{xml}->elements_by_name("VALUE");
	my $key;

	my $i = 0;
	my $numKeys = @keys;
	my $numValues = @values;
	if ($numKeys != $numValues)
	{
		die "Invalid data in RC, please contact Telelogic technical support";
	}

	#Basic information about Object RC
	foreach $key (@keys)
	{
		my $value = $values[$i];
		my $keyContent = $key->get_content();
		my $valueContent = $value->get_content();
		$i++;	
		if ($keyContent eq "rcID")
		{
			$self->{rcID} = $valueContent;
		}
		elsif ($keyContent eq "parent_rcrID")
		{
			$self->{parent_rcrID} = $valueContent;
		}
		elsif ($keyContent eq "doorsID")
		{
			$self->{doorsID} = $valueContent;
		}
		elsif ($keyContent eq "doors_object_path")
		{
			$self->{doors_object_path} = $valueContent;
		}
		elsif ($keyContent eq "doorsIsApplied")
		{
			$self->{doorsIsApplied} = $valueContent;
		}
		elsif ($keyContent eq "doorsIsRejected")
		{
			$self->{doorsIsRejected} = $valueContent;
		}
	}

	#List of modified attributes
	my $modifyAttrsSection = $self->{xml}->elements_by_name("MODIFY_ATTRS");
	my $modifyAttrsSectionContent = $modifyAttrsSection->get_content();
	chomp $modifyAttrsSectionContent;
	my $modifyAttrs = new XML::Lite($modifyAttrsSectionContent);
	if (defined $modifyAttrs)
	{
		my @attrNames = $modifyAttrs->elements_by_name("NAME");
		my $attrName;
		my @listOfModifiedAttrs;
		foreach $attrName (@attrNames)
		{
			push(@listOfModifiedAttrs, $attrName->get_content());
		}
		$self->{modify_attrs} = \@listOfModifiedAttrs;
	}

	#Actions
	my @actionSections = $self->{xml}->elements_by_name("ACTION");
	my $actionSection;
	my @listOfActions;
	foreach $actionSection (@actionSections)
	{	
		my $actionSectionContent = $actionSection->get_content();
		chomp $actionSectionContent;
		my $actions = new XML::Lite($actionSectionContent);
		if (defined $actions)
		{
			my $actionName = $actions->elements_by_name("NAME");

#			print $self->{parent_rcrID} . ": ";
#			print $self->{doorsID} . "   ";
			$actionName = $actionName->get_content();
#			print $actionName . "\n";
			push(@listOfActions, $actionName);
			if (($actionName eq $RC_ACTION_MOVE) || ($actionName eq $RC_ACTION_ADD))
			{
				$self->{after} = $actions->elements_by_name("After")->get_content();
				$self->{below} = $actions->elements_by_name("Below")->get_content();
				$self->{siblings} = $actions->elements_by_name("Siblings")->get_content();
			}
			my @linkData;
			if (($actionName eq $RC_ACTION_REMOVE_OUTLINKS) || ($actionName eq $RC_ACTION_CREATE_OUTLINKS))
			{
				my @outlinks = $actions->elements_by_name("LinkData");
				foreach my $outlink (@outlinks)
				{
					push(@linkData, $outlink->get_content());
				}
				$self->{linkdata} = \@linkData;
			}
		}
	}
	$self->{actions} = \@listOfActions;
}

1;
