// Copyright (c) 2002-2007 Telelogic AB
// All rights reserved.

/*
  <trigger>
	<usage>bsf verify_related_change_request_states.js relationship states</usage>
	<version>5.0</version>
	<description>
		在转变 CR 时，验证相关的变更请求是否处于某种指定的状态。如果任何 CR 未处于这些状态之一，该触发器将失败。用作转变前触发器时，这将使转变不可访问。</description>
	<argument1>
		<name>relationship</name>
		<required>yes</required>
		<type>string</type>
		<description>用于获取相关 CR 的关系的名称。例如，如果要使用一个流程，通过“child_problem”关系将父 CR 与子 CR 关联，那么使用“child_problem”。</description>
	</argument1>
	<argument2>
		<name>states</name>
		<required>yes</required>
		<type>string</type>
		<description>要检查的以空格分隔的状态名称列表。未处于这些状态之一的相关 CR 将导致触发器失败。</description>
	</argument2>
</trigger>
*/
var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');

if (args.length < 2)
{
	log.error('Could not verify related change requests: missing arguments.');
	log.error('Usage: verify_related_change_request_states.js relationship state [or_state...]');
}
else
{
	// Since java array from Rhino is of immutable length, we need to use slice to convert it to a regular javascript array
	var jsArgs = args.slice();
	var relationship = jsArgs.shift();
	var expectedStates = jsArgs;
	var expectedStateSet = new java.util.HashSet(java.util.Arrays.asList(expectedStates));

	var related = change.getRelatedFrom(event.adminUser.token, event.objectId, relationship, ['crstatus']);

	for (var i = 0; i < related.length; i++)
	{
		var currentCr = related[i];
		var actualState = currentCr.getAttribute('crstatus').stringValue;

		if (!expectedStateSet.contains(actualState))
		{
			event.stopTransition(formatError(expectedStates, relationship));
		}
	}
}

function formatError(states, relationship)
{
	if (states.length == 1)
	{
		return change.getMessage('notAllRelationshipXCrsInStateX', [relationship, states[0]]);
	}
	else
	{
		return change.getMessage('notAllRelationshipXCrsInStateListX', [relationship, states.join(', ')]);
	}
}